<?php


namespace OCA\GrueneGroupfolderManagement\Cache;

use OCP\ICache;

class RandomizeTtlDecorator implements ICache {
	/**
	 * @var ICache
	 */
	protected $cache;

	/**
	 * @var int
	 */
	protected $randomFactor = 1;

	public function __construct(ICache $cache, float $randomFactor = 1) {
		$this->cache = $cache;
		$this->randomFactor = $randomFactor;
	}

	public function get($key) {
		return $this->cache->get($key);
	}

	public function set($key, $value, $ttl = 0) {
		if ($ttl === null) {
			$ttl = 0;
		}

		if ($ttl !== 0) {
			$random = ceil($ttl * $this->randomFactor);
			$ttl = $ttl + rand(0, $random);
		}

		return $this->cache->set($key, $value, $ttl);
	}

	public function hasKey($key) {
		return $this->cache->hasKey($key);
	}

	public function remove($key) {
		return $this->cache->remove($key);
	}

	public function clear($prefix = '') {
		return $this->cache->remove($prefix);
	}

	public static function isAvailable(): bool {
		return true;
	}
}
