<?php
/**
 * Circles - Bring cloud-users closer together.
 *
 * This file is licensed under the Affero General Public License version 3 or
 * later. See the COPYING file.
 *
 * @author Maxence Lange <maxence@artificial-owl.com>
 * @copyright 2017
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

namespace OCA\GrueneGroupfolderManagement\Command;

use OC\Core\Command\Base;
use OCA\GrueneGroupfolderManagement\Constants\Constants;
use OCA\GrueneGroupfolderManagement\Db\GroupMapper;
use OCA\GrueneGroupfolderManagement\Manager\DivisionManager;
use OCA\GrueneGroupfolderManagement\Manager\DirectoryManager;
use OCA\GrueneGroupfolderManagement\Model\Division;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class EnsureDirectoriesCommand extends Base {
	/**
	 * @var DirectoryManager
	 */
	protected $directoryManager;

	/**
	 * @var DivisionManager
	 */
	protected $divisionManager;

	/**
	 * @var GroupMapper
	 */
	protected $groupMapper;

	public function __construct(
		DirectoryManager $directoryManager,
		DivisionManager $divisionManager,
		GroupMapper $groupMapper
	) {
		parent::__construct();
		$this->directoryManager = $directoryManager;
		$this->divisionManager = $divisionManager;
		$this->groupMapper = $groupMapper;
	}

	protected function configure() {
		parent::configure();
		$this->setName('gruenegroupfoldermanagement:ensure')
			->addArgument('divisions', InputArgument::IS_ARRAY | InputArgument::OPTIONAL)
			->setDescription('iterates over all divisions and creates directories');
	}

	protected function findDivisionsByIds(array $divisionIds) : iterable {
		foreach ($divisionIds as $divisionId) {
			yield $this->divisionManager->find($divisionId);
		}

		yield from [];
	}

	protected function execute(InputInterface $input, OutputInterface $output) {
		$divisionIds = $input->getArgument('divisions');

		if (empty($divisionIds)) {
			$divisions = $this->divisionManager->findAll();
		} else {
			$divisions = $this->findDivisionsByIds($divisionIds);
		}

		/** @var Division */
		foreach ($divisions as $division) {
			$output->write('<info>Ensure '.$division->getId().' => '.$division->getOrganization()->getName().' / '.$division->getName().'... </info>');
			try {
				if ($division->getType() !== Constants::RV_Type) {
					// skip regional divisions
					$groups = $this->groupMapper->findByDivision($division->getOrganizationalUuid());
					$this->directoryManager->saveToFileSystem($groups, $division, true);
					$output->writeln('<info>[DONE]</info>');
				} else {
					$output->writeln('<info>[SKIPPED]</info>');
				}
			} catch (\Throwable $e) {
				$output->writeln('<error>[ERROR] => '.$e->__toString().'</error>');
			}
		}
	}
}
