<?php

namespace OCA\GrueneGroupfolderManagement\Controller;

use OCA\GrueneGroupfolderManagement\AppInfo\Application;
use OCA\GrueneGroupfolderManagement\Exceptions\AccessDeniedException;
use OCA\GrueneGroupfolderManagement\Exceptions\ValidationFailedException;
use OCA\GrueneGroupfolderManagement\Security\AuthorizationService;
use OCA\GrueneGroupfolderManagement\Validation\ValidatorService;
use OCP\AppFramework\Controller;
use OCP\IRequest;

class BaseController extends Controller {
	private AuthorizationService $authorizationService;
	private ValidatorService $validatorService;

	public function __construct(
	) {
		parent::__construct(
			Application::APP_ID,
			\OC::$server->get(IRequest::class),
		);

		$this->authorizationService = \OC::$server->get(AuthorizationService::class);
		$this->validatorService = \OC::$server->get(ValidatorService::class);
	}

	/**
	 * Throws an exception if the payload is not valid for the given DTO class
	 *
	 * @throws ValidationFailedException
	 */
	protected function validate(array $input, string $dtoClass) {
		return $this->validatorService->validateAndCreate($input, $dtoClass);
	}

	protected function createValidationException(string $message) {
		throw new ValidationFailedException($message, []);
	}

	/**
	 * Throws an exception unless the attributes are granted for the current authentication user and optionally
	 * supplied subject.
	 *
	 * @param string[] $attributes The attributes
	 * @param mixed    $subject    The subject
	 * @param string[] $attributes Attributes of subject
	 * @param string   $message    The message passed to the exception
	 *
	 * @throws AccessDeniedException
	 */
	protected function denyAccessUnlessGranted(array $attributes, $subject, $message = 'Access Denied.') {
		if (!$this->authorizationService->isGranted($attributes, $subject)) {
			throw new AccessDeniedException($message);
		}
	}
}
