<?php

namespace OCA\GrueneGroupfolderManagement\DTO;

use Respect\Validation\Validator as v;

class UpdateGroupDto {
	public function __construct(
		public ?string $name = null,
		public ?string $type = null,
		public ?int $divisionAclMask = null,
		public ?int $divisionAclPermission = null,
		public ?int $memberAclMask = null,
		public ?int $memberAclPermission = null,
		public ?int $adminAclMask = null,
		public ?int $adminAclPermission = null,
		public ?bool $active = null,
	) {
	}

	public static function getValidator(): v {
		return v::create()
			->key('name', v::stringType()->length(3, 255), false)
			->key('type', v::stringVal()->oneOf(
				v::equals('OTHER'),
				v::equals('WORKGROUP'),
				v::equals('EXECUTIVE_BOARD'),
			)->setTemplate('must be one of OTHER, WORKGROUP or EXECUTIVE_BOARD'), false)
			->key('divisionAclMask', v::intVal()->between(0, 31), false)
			->key('divisionAclPermission', v::intVal()->between(0, 31), false)
			->key('memberAclMask', v::intVal()->between(0, 31), false)
			->key('memberAclPermission', v::intVal()->between(0, 31), false)
			->key('adminAclMask', v::intVal()->between(0, 31), false)
			->key('adminAclPermission', v::intVal()->between(0, 31), false)
			->key('active', v::boolType(), false)
		;
	}
}
