<?php


namespace OCA\GrueneGroupfolderManagement\Groups;

use OCP\Group\Backend\ABackend;
use OCP\ICache;

class CachedGroupBackendDecorator extends ABackend {
	/**
	 * @var ICache
	 */
	protected $cache;

	/**
	 * @var ABackend
	 */
	protected $backend;

	public function __construct(ICache $cache, ABackend $backend) {
		$this->cache = $cache;
		$this->backend = $backend;
	}

	public function getBackend() : ABackend {
		return $this->backend;
	}

	protected function cached($callback, string $prefix, array $parameters) {
		$hash = 'groupbackend_' . hash('sha256', get_class($this->backend) . '_' . $prefix . '_' . \json_encode($parameters));

		$result = $this->cache->get($hash);

		if ($result instanceof \stdClass && !property_exists($result, 'item')) {
			$result = null;
		}

		if (!($result instanceof \stdClass)) {
			$result = new \stdClass();
			$result->item = call_user_func_array($callback, $parameters);
			$this->cache->set($hash, $result);
		}

		return $result->item;
	}

	public function inGroup($uid, $gid) {
		return $this->cached(function ($uid, $gid) {
			return $this->backend->inGroup($uid, $gid);
		}, 'inGroup', [$uid, $gid]);
	}

	public function getUserGroups($uid) {
		return $this->cached(function ($uid) {
			return $this->backend->getUserGroups($uid);
		}, 'getUserGroups', [$uid]);
	}

	public function getGroups($search = '', $limit = -1, $offset = 0) {
		return $this->cached(function ($search, $limit, $offset) {
			return $this->backend->getGroups($search, $limit, $offset);
		}, 'getGroups', [$search, $limit, $offset]);
	}

	public function groupExists($gid) {
		return $this->cached(function ($gid) {
			return $this->backend->groupExists($gid);
		}, 'groupExists', [$gid]);
	}

	public function usersInGroup($gid, $search = '', $limit = -1, $offset = 0) {
		return $this->cached(function ($gid, $search, $limit, $offset) {
			return $this->backend->usersInGroup($gid, $search, $limit, $offset);
		}, 'usersInGroup', [$gid, $search, $limit, $offset]);
	}
}
