<?php

namespace OCA\GrueneGroupfolderManagement\Manager;

use OCA\GrueneGroupfolderManagement\ApiClient;
use OCA\GrueneGroupfolderManagement\Converter\DivisionConverter;
use OCA\GrueneGroupfolderManagement\Converter\OrganizationConverter;
use OCA\GrueneGroupfolderManagement\Model\Division;
use OCA\GrueneGroupfolderManagement\Model\Organization;

class DivisionManager {
	/**
	 * @var ApiClient
	 */
	protected $client;

	/**
	 * @var DivisionConverter
	 */
	protected $divisionConverter;

	/**
	 * @var OrganizationConverter
	 */
	protected $organizationConverter;

	public function __construct(
		ApiClient $client,
		DivisionConverter $divisionConverter,
		OrganizationConverter $organizationConverter
	) {
		$this->client = $client;
		$this->divisionConverter = $divisionConverter;
		$this->organizationConverter = $organizationConverter;
	}

	/**
	 * @return Organization[]
	 */
	public function getAllOrganizations() {
		// TODO add cache

		$list = $this->client->partyOrganizations();

		$result = [];

		foreach ($list['items'] as $item) {
			$result[] = $this->organizationConverter->convertToEntity(
				$item
			);
		}

		return $result;
	}

	/**
	 * @return Organization|null
	 */
	public function findOrganization($id) {
		// TODO add cache
		// TODO optimize

		foreach ($this->getAllOrganizations() as $organization) {
			if ($organization->getId() == $id) {
				return $organization;
			}
		}

		return null;
	}

	/**
	 * @param string $search
	 * @param string $prefixId
	 * @param string $type
	 *
	 * @return Division[]
	 */
	public function findAll($search = null, $prefixId = null, $type = null) {
		// TODO support subdivisions
		// TODO add cache

		$list = $this->client->partyRegionalchapters($search, $prefixId, $type);

		$result = [];

		foreach ($list['items'] as $item) {
			$result[] = $this->divisionConverter->convertToEntity(
				$item
			);
		}

		return $result;
	}

	/**
	 * @param string $id
	 *
	 * @return Division|null
	 */
	public function find($id) {
		// TODO support subdivisions
		// TODO add cache

		$data = $this->client->partyRegionalchapter($id);

		if ($data !== null) {
			return $this->divisionConverter->convertToEntity(
				$data
			);
		}

		return null;
	}
}
