<?php

namespace OCA\GrueneGroupfolderManagement\Model;

class Organization implements OrganizationalUnitInterface, \JsonSerializable {
	/**
	 * @var string|null
	 */
	protected $name;

	/**
	 * @var string|null
	 */
	protected $id;

	/**
	 * @var string|null
	 */
	protected $regionalChapterIdPrefix;

	public function getOrganizationalUuid() {
		return str_pad($this->getRegionalChapterIdPrefix(), 8, 0);
	}

	public function getOrganizationalQuery() {
		return null;
	}

	public function getOrganizationalName() {
		return $this->getName();
	}

	/**
	 * @return string|null
	 */
	public function getId(): ?string {
		return $this->id;
	}

	/**
	 * @param string|null $id
	 */
	public function setId(?string $id): void {
		$this->id = $id;
	}

	/**
	 * @return string|null
	 */
	public function getName(): ?string {
		return $this->name;
	}

	/**
	 * @param string|null $name
	 */
	public function setName(?string $name): void {
		$this->name = $name;
	}

	/**
	 * @return string|null
	 */
	public function getRegionalChapterIdPrefix(): ?string {
		return $this->regionalChapterIdPrefix;
	}

	/**
	 * @param string|null $regionalChapterIdPrefix
	 */
	public function setRegionalChapterIdPrefix(?string $regionalChapterIdPrefix): void {
		$this->regionalChapterIdPrefix = $regionalChapterIdPrefix;
	}

	/**
	 * Specify data which should be serialized to JSON
	 * @link https://php.net/manual/en/jsonserializable.jsonserialize.php
	 * @return mixed data which can be serialized by <b>json_encode</b>,
	 * which is a value of any type other than a resource.
	 * @since 5.4.0
	 */
	public function jsonSerialize(): mixed {
		return [
			'id' => $this->id,
			'name' => $this->name,
			'regional_chapter_id_prefix' => $this->regionalChapterIdPrefix
		];
	}
}
