<?php

namespace OCA\GrueneGroupfolderManagement\Security;

use OCA\GrueneGroupfolderManagement\Manager\SherpaManager;
use OCA\GrueneGroupfolderManagement\Model\Division;
use OCP\IUser;

class DivisionVoter extends Voter {
	public function __construct(
		private SherpaManager $sherpaManager,
	) {
	}
	protected function supports(string $attribute, mixed $subject): bool {
		return $subject instanceof Division;
	}

	protected function voteOnAttribute(string $attribute, mixed $subject, ?IUser $user): bool {
		if (!$user) {
			return false;
		}

		/** @var Division */
		$division = $subject;

		return match ($attribute) {
			'CREATE_GROUP',
			'READ_GROUP',
			'UPDATE_GROUP',
			'DELETE_GROUP' => $this->isGrantedDivision($user, $division),
			default => throw new \LogicException('This code should not be reached!')
		};
	}

	protected function isGrantedDivision(IUser $user, Division $division) : bool {
		foreach ($this->sherpaManager->filterDivisionRoles($user, 'TECHNICAL_WOLKE') as $divisionGroup) {
			if ($division->getOrganizationalUuid() == $divisionGroup->getDivision()->getOrganizationalUuid()) {
				return true;
			}
		}

		return false;
	}
}
