<?php

namespace OCA\GrueneGroupfolderManagement\Security;

use OCA\GrueneGroupfolderManagement\Db\Group;
use OCA\GrueneGroupfolderManagement\Manager\SherpaManager;
use OCA\GrueneGroupfolderManagement\Model\Division;
use OCP\IUser;

class GroupVoter extends Voter {
	public function __construct(
		private SherpaManager $sherpaManager,
	) {
	}
	protected function supports(string $attribute, mixed $subject): bool {
		return $subject instanceof Group || $subject === Group::class;
	}


	protected function voteOnAttribute(string $attribute, mixed $subject, ?IUser $user): bool {
		// _dlog($attribute, $subject);

		if (!$user) {
			return false;
		}

		// if ($subject instanceof Group) {

		// }

		/** @var Group */
		$group = $subject;
		return match ($attribute) {
			'CREATE' => $this->isGrantedDivision($user, $group->getDivision()),
			'READ' => $this->isGrantedDivision($user, $group->getDivision()),
			'UPDATE' => $this->isGrantedDivision($user, $group->getDivision()),
			'DELETE' => $this->isGrantedDivision($user, $group->getDivision()),
			default => throw new \LogicException('This code should not be reached!')
		};
	}

	/**
	 * @param IUser $user
	 * @return Division[]
	 */
	private function getAdministerableDivisions(IUser $user) : \Traversable {
		foreach ($this->sherpaManager->filterDivisionRoles($user, 'TECHNICAL_WOLKE') as $divisionGroup) {
			yield $divisionGroup->getDivision();
		}

		yield from [];
	}

	protected function isGrantedDivision(IUser $user, Division $division) : bool {
		// TODO better performance
		foreach ($this->getAdministerableDivisions($user) as $granted) {
			if ($division->getOrganizationalUuid() == $granted->getOrganizationalUuid()) {
				return true;
			}
		}

		return false;
	}
}
